@@ -1,4 +1,4 @@
-/* $Id: miniupnpcmodule.c,v 1.32 2018/05/03 08:17:48 nanard Exp $*/
+/* $Id: miniupnpcmodule.c,v 1.33 2019/02/10 13:22:47 nanard Exp $*/
 /* Project : miniupnp
  * Author : Thomas BERNARD
  * website : https://miniupnp.tuxfamily.org/
@@ -676,6 +676,16 @@ initminiupnpc(void)
     /* initialize Winsock. */
     WSADATA wsaData;
     int nResult = WSAStartup(MAKEWORD(2,2), &wsaData);
+	if (nResult != 0)
+	{
+		/* error code could be WSASYSNOTREADY WSASYSNOTREADY
+		 * WSASYSNOTREADY WSASYSNOTREADY WSASYSNOTREADY */
+#if PY_MAJOR_VERSION >= 3
+        return 0;
+#else
+        return;
+#endif
+	}
 
     UPnPType.tp_new = PyType_GenericNew;
 #endif
